<?php

namespace Intuit\Jobs;

use Carbon\Carbon;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

use Intuit\Models\{Invoice,ProviderToken};
use App\Models\Invoice as AppInvoice;

class AccountingInvoiceUpdate implements ShouldQueue
{
	use Dispatchable,InteractsWithQueue,Queueable,SerializesModels;

	private const LOGKEY = 'JIU';

	private Invoice $o;
	private ProviderToken $to;

	/**
	 * Create a new job instance.
	 *
	 * @return void
	 */
	public function __construct(ProviderToken $to,Invoice $item)
	{
		$this->onQueue('intuit');

		$this->o = $item;
		$this->to = $to->withoutRelations();
	}

	/**
	 * Execute the job.
	 *
	 * @return void
	 */
	public function handle()
	{
		$api = $this->to->API();

		// Get the original invoice, so we can get the provider sync details
		$o = $this->to->provider->invoices->where('id',$this->o->DocNumber);
		$o = ($o->count() === 1) ? $o->pop() : AppInvoice::findOrFail($this->o->DocNumber);

		$updated = $api->updateInvoice(array_merge(
			$this->o->getDirty(),
			$o->providers->count()
				?
				[
					'Id'=>$o->pivot->ref,
					'SyncToken'=>$o->pivot->synctoken,
				]
				:
				[]
		));

		if ($updated instanceof \Intuit\Response\Invoice) {
			$o->providers()->syncWithoutDetaching([
				$this->to->provider->id => [
					'ref' => $updated->id,
					'synctoken' => $updated->synctoken,
					'created_at'=>Carbon::create($updated->created_at),
					'updated_at'=>Carbon::create($updated->updated_at),
					'site_id'=>$o->site_id,						 // @todo See if we can have this handled automatically
				],
			]);

			Log::info(sprintf('%s:Updated invoice [%s:%s] (%s:%s), synctoken now [%s]',self::LOGKEY,$this->to->provider->name,$o->sid,$updated->id,$updated->DocNumber,$updated->synctoken));

		} else {
			Log::error(sprintf('%s:Unable to update invoice with provider [%s] for [%s:%s]',self::LOGKEY,$this->to->provider->name,$updated->id,$updated->DisplayName),['updated'=>get_class($updated)]);
		}
	}
}